/*
	uart_session.c --- user session via UART (Serial) interface functions
*/

#include <msp430.h>
#include <stdio.h>
#include <ctype.h>

#include "user_define.h"

/* -------- global var. -------- */
unsigned char CMDLINE[CMDLINE_BUFSIZE];


/* check a char within '0'-'9' 'a'-'f' 'A'-'F' */
BOOL isHexChar(unsigned char c){
	BOOL ret;
	ret = FALSE;
	if ((c>='0') && (c<='9'))  ret = TRUE;
	c = toupper(c);
	if ((c>='A') && (c<='F'))  ret = TRUE;
	return ret;
}

/* hex string to int (up to 4digit)*/
/* pointer *str should point the first letter of hex string, and */
/* should terminate null(0) or Space. */
unsigned int hex2val(unsigned char *str)
{
	unsigned int ret,m,p;
	p=0;
	ret=0;
	while ((str[p]!=0)&&(str[p]!=SPC)&&(p<5)){ p++; }	// forward to delimiter
	if (p>=5){ return 0; }
	m=1;
	while (p>0){
		p--;
		if (toupper(str[p]) >= 'A'){
			ret += m * ((toupper(str[p])-'A') + 10);
		} else {
			ret += m * (str[p]-'0');
		}
		m = m << 4;
	}
	return ret;
}
/* int to hex string (up to 4digit)*/
void hexstr(unsigned int val, char *str)
{
	char *hextbl="0123456789ABCDEF?";
	str[0] = hextbl[val >> 12];
	str[1] = hextbl[(val >> 8) & 0x000f];
	str[2] = hextbl[(val >> 4) & 0x000f];
	str[3] = hextbl[val & 0x000f];
	str[4] = 0;
}

/* ---------- Terminal session functions ---------- */
void show_version()
{
	uart_sendstr("\r\nI2C bus device access tool  ver 0.1 2013 (C) espilab\r\n");
}

void show_help()
{
	char *cp =
		" a <adr(hex)>   : set slave address\r\n"
		" s xx [xx] ..   : set byte(s) data to write\r\n"
		" c <bytes(hex)> : set count bytes to read\r\n"
		" r              : read access\r\n"
		" w              : write access\r\n"
		" p              : show current parameters\r\n"
		" 0              : reset i2c I/F\r\n"
		" P              : output stop condition\r\n"
		" V              : show version\r\n"
		" ?              : show help\r\n"
		"read/write bytes count is 64 bytes max.\r\n";
	uart_sendstr(cp);
}


/* --- get one command line --- */
/*   return  >=0 : length (enter key pressed)
             <0  : no any key hit
     always through and return
*/
int cmd_line(unsigned char *cmdlineptr)
{
	unsigned char cin;
	int len;
	BOOL enterkey;
	static int cptr = 0;

	enterkey = FALSE;
	while(recv_len() > 0){
		if (cin = uart_getchar()){
			if (cin == BS){
				if (cptr >0){
					uart_sendchar(BS);
					uart_sendchar(' ');
					uart_sendchar(BS);
					cptr--;
				} else {
					uart_sendchar(BEL);
				}
			}
			else if (cin == CR){
				uart_sendchar(CR);
				uart_sendchar(LF);
				cmdlineptr[cptr] = 0;
				len = cptr;
				cptr = 0;
				enterkey = TRUE;
			}
			else {
				if (cptr < (CMDLINE_BUFSIZE-1)){
					uart_sendchar(cin);
					cmdlineptr[cptr++] = cin;
				} else {
					uart_sendchar(BEL);
				}
			}
		}
	} /* while(.. */
	if (enterkey){
		return len;
	} else {
		return -1;
	}
}

/* decode and process command */
void process_command(unsigned char *cmdline)
{
	int j,cnt, ret;
	static int txlen=1, rxlen=1;
	unsigned char *cp;
	char str[8];
	unsigned int adr,data;

	switch (cmdline[0]){
		case '?':
			show_help();
			break;
		case 'a':			// set slave address
			cp = ++cmdline;
			while ((*cp!=NULL) && !isHexChar(*cp)) cp++;
			if (*cp == NULL){
				uart_sendstr("?\r\n");
			} else {
				adr = hex2val(cp);
				I2C_SLAVE_ADR = adr & 0x00ff;
				hexstr(I2C_SLAVE_ADR, str);
				uart_sendstr("Set slave address = 0x");
				uart_sendstr(&str[2]);
				uart_sendstr("\r\n");
			}
			break;
		case 'c':			// set read bytes
			cp = ++cmdline;
			while ((*cp!=NULL) && !isHexChar(*cp)) cp++;
			if (*cp == NULL){
				uart_sendstr("?\r\n");
			} else {
				j = hex2val(cp);
				if (j > I2C_BUFSIZE){
					uart_sendstr("over range\r\n");
				} else {
					rxlen = j;
					hexstr(rxlen, str);
					uart_sendstr("read counter = 0x");
					uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
				}
			}
			break;
		case 's':			// set write data array
			cp = ++cmdline;
			while ((*cp!=NULL) && !isHexChar(*cp)) cp++;
			if (*cp == NULL){
				uart_sendstr("?\r\n");
			} else {
				cnt = 0;
				while (cnt < I2C_BUFSIZE){
					data = hex2val(cp);
					I2C_BUF_W[cnt++] = (unsigned char)(data & 0x00ff);
					while ((*cp!=NULL) && isHexChar(*cp)) cp++;	// skip current data str
					while ((*cp!=NULL) && !isHexChar(*cp)) cp++;	// skip space
					if (*cp == NULL) break;
				}
				txlen = cnt;
				uart_sendstr("data for write (hex):");
				for (j=0; j<txlen; j++){
					hexstr(I2C_BUF_W[j], str);
					uart_sendstr(&str[2]);
					uart_sendstr(" ");
				}
				uart_sendstr(",  0x");
				hexstr(txlen, str);
				uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
			}
			break;
		case 'p':			// show current parameters
			hexstr(I2C_SLAVE_ADR, str);
			uart_sendstr("slave address = 0x");
			uart_sendstr(&str[2]);
			uart_sendstr("\r\n");
			hexstr(rxlen, str);
			uart_sendstr("read counter = 0x");		uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
			uart_sendstr("write data: ");
			for (j=0; j<txlen; j++){
				hexstr(I2C_BUF_W[j], str);
				uart_sendstr(&str[2]);
				uart_sendstr(" ");
			}
			uart_sendstr(",  0x");
			hexstr(txlen, str);			uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
			break;
		case 'w':			// execute i2c write
			I2C_TxByteCnt = txlen;
			ret = i2c_send(I2C_SLAVE_ADR, I2C_BUF_W, I2C_TxByteCnt);
			if (ret == E_BUSY)		uart_sendstr("BUSY\r\n");
			if (ret == E_NACK)		uart_sendstr("NACK\r\n");
			if (ret == E_TMOUT)		uart_sendstr("Time out\r\n");
			break;
		case 'r':			// execute i2c read
			I2C_RxByteCnt = rxlen;
			ret = i2c_recv(I2C_SLAVE_ADR, I2C_BUF_R, I2C_RxByteCnt);
			if (ret == E_BUSY)		uart_sendstr("BUSY\r\n");
			if (ret == E_NACK)		uart_sendstr("NACK\r\n");
			if (ret == E_TMOUT)		uart_sendstr("Time out\r\n");
			if (ret > 0){
				for (j=0; j<ret; j++){
					hexstr(I2C_BUF_R[j], str);
					uart_sendstr(&str[2]);
					uart_sendstr(" ");
					if (((j+1) & 0x000f) == 0)	uart_sendstr("\r\n");
				}
				uart_sendstr("\r\n");

			}
			break;
		case '0':			// i2c reset
			i2c_reset();
			uart_sendstr("I2C reset\r\n");
			break;
		case 'P':			// i2c stop condition
			i2c_stop();
			uart_sendstr("stop condition\r\n");
			break;
		case 'V':			// show version
			show_version();
			break;
		default:
				break;
	}
}

/* Session --- if no received data, then through and return */
void session()
{

	if (cmd_line(CMDLINE) >= 0){
		process_command(CMDLINE);
		uart_sendchar('>');
	}
	else {
		;		// quit immediately.
	}
}
